/*
 *      relay-nc800.c
 *
 *     	multipart/mixed Relay for Gadspot nc-800
 *
 *	Some source code has been taken from motion ( webhttpd.c , netcam.c and webcam.c )
 *	
 *	
 *      Copyright 2004-2005 by Angel Carpintero  (ack@telefonica.net)
 *      This software is distributed under the GNU Public License Version 2
 *      See also the file 'COPYING'.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#define __USE_GNU
#include <string.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>

#define VERSION "0.7"
#define MAX_IMAGE_BUFFER 655360 // 512 Kb 
#define MAX_RETRY 3 

/* How many bytes it will take to store LEN bytes in base64.  */
#define BASE64_LENGTH(len) (4 * (((len) + 2) / 3))

#define SLEEP(seconds, nanoseconds) {              \
                struct timespec tv;                \
                tv.tv_sec = (seconds);             \
                tv.tv_nsec = (nanoseconds);        \
                while (nanosleep(&tv, &tv) == -1); \
        }

int  port_relay;
char *host_camera=NULL;
char *url_path=NULL;
int port_camera;
char *get_request=NULL;

const char *get = "GET %s HTTP/1.0\r\n"
		  "User-Agent: Motion-Relay \"VERSION\"\r\n"	
		  "Connection: close\r\n";

static const char *header = "HTTP/1.0 200 OK\r\n"
                        "Server: Motion-Relay\r\n"
                        "Connection: close\r\n"
                        "Max-Age: 0\r\n"
                        "Expires: 0\r\n"
                        "Cache-Control: no-cache, private\r\n"
                        "Pragma: no-cache\r\n"
                        "Content-Type: multipart/x-mixed-replace; boundary=--BoundaryString";

static const char *jpeghead = "\r\n\r\n--BoundaryString\r\n"
                       "Content-type: image/jpeg\r\n";


// Auth for cameras that requiere

const char *connect_auth_req = "Authorization: Basic %s\r\n";		       
char *auth=NULL;

// TODO : move GLOBAL to LOCAL variables or at least to a struct

size_t size_header;
char image_buffer[MAX_IMAGE_BUFFER]={'\0'};  
char *jpeg_image = NULL; // To store image to send to clients
/* Current used size of image_buffer */
size_t buffer_pos = 0;
/* Size of image in image_buffer */
long image_size = 0;


/* JPEG image flags */
int image_complete = 0;
int start_jpg = 0;
int end_jpg = 0;

/* Stream flags */ 
int start_stream = 1;

/* debug flag */
int debug = 0;

/*
 * Write the ini header for multipart/mixed stream
 *
 * Return 0 on error
 * 	  1 on success
 * 	  
 */ 
static int write_ini_header(int clientsocket)
{
	ssize_t nsend;
	nsend = send(clientsocket,header,strlen(header),0);
	if (nsend <= 0){
		if (nsend == -1){
			printf("write_ini_header %s\n",strerror(errno));		
		}
		return -1;	
	}
	start_stream = 0;
	
	return 1;
}


/*
 *
 * Write to client the image
 *
 * Return 0 on error
 * 	  1 on success
 * 
 */ 
static int write_client(int clientsocket)
{
	ssize_t nsend,sendb = 0;
	char *content_length = NULL;
	int content_length_len;
	
	content_length = malloc(30);
	
	content_length_len = sprintf(content_length,"Content-Length: %ld\r\n\r\n",image_size);
	
	
	do{
		nsend = send(clientsocket,jpeghead,size_header-sendb,0);
		if (nsend == -1){
			printf("write_client jpegheader %s\n",strerror(errno));
			if ((errno == EINTR) || (errno == EAGAIN )){ 
				SLEEP(0,10000);
				continue;
			}
				return 0;
		}else sendb += nsend;		
                   	
	}while (sendb < (ssize_t)size_header);

	sendb = 0;

	send(clientsocket,content_length,content_length_len,0);

	free(content_length);
	
	do{
		nsend = send(clientsocket,jpeg_image,image_size-sendb,0);
		if (nsend == -1){
			printf("write_client jpeg_image %s %d\n",strerror(errno),errno);
			if ((errno == EINTR) || (errno == EAGAIN )){
				SLEEP(0,10000); 
				continue;
			}
				return 0;
                }else sendb += nsend;
		
	}while (sendb < image_size);

	return 1;
}

/*
 *         Encode the string S of length LENGTH to base64 format and place it
 *         to STORE.  STORE will be 0-terminated, and must point to a writable
 *         buffer of at least 1+BASE64_LENGTH(length) bytes.
 */
static void base64_encode(const char *s, char *store, int length)
{
        /* Conversion table.  */
        static const char tbl[64] = {
                'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
                'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
                'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
                'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
                'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
                'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
                'w', 'x', 'y', 'z', '0', '1', '2', '3',
                '4', '5', '6', '7', '8', '9', '+', '/'
        };

        int i;
        unsigned char *p = (unsigned char *)store;

        /* Transform the 3x8 bits to 4x6 bits, as required by base64.  */
        for (i = 0; i < length; i += 3) {
                *p++ = tbl[s[0] >> 2];
                *p++ = tbl[((s[0] & 3) << 4) + (s[1] >> 4)];
                *p++ = tbl[((s[1] & 0xf) << 2) + (s[2] >> 6)];
                *p++ = tbl[s[2] & 0x3f];
                s += 3;
        }

        /* Pad the result if necessary...  */
        if (i == length + 1)
                *(p - 1) = '=';
        else if (i == length + 2)
                *(p - 1) = *(p - 2) = '=';

        /* ...and zero-terminate it.  */
        *p = '\0';
}


/*
 * Send Basic authentication  
 *
 * Return 0 on error
 * 	  1 on success
 *
 */ 
static int send_auth(int camerasocket)
{
	int sendb=0;
	char *encuserpass = NULL;
	char *request_pass = NULL;

	/* Allocate space for the base64-encoded string */
        encuserpass = malloc(BASE64_LENGTH(strlen(auth)) + 1);
        /* Fill in the value */
        base64_encode(auth, encuserpass, strlen(auth));
        /* Now create the last part (authorization) of the request */
        request_pass = malloc(strlen(connect_auth_req) + strlen(encuserpass) + 1);
        sprintf(request_pass, connect_auth_req, encuserpass);
        /* free the working variables */
        free(encuserpass);
	sendb = send(camerasocket,request_pass,strlen(request_pass),0);

	if (sendb < 0){
		printf("Error sending 'auth' request\n");
		printf("send_request %s\n",strerror(errno));
		return 0;
        }
	
	printf("send_auth [OK]\n");
	
	return 1;
}

/*
 * Send HTTP GET request to get camera stream
 *
 * Return 0 on error
 * 	  1 on success
 * 	  
 */ 
static int send_request(int camerasocket)
{
	int sendb=0;

	if (camerasocket == -1){
		printf("Not valid socket\n");
		return 0;
	}	

	// Send HTTP GET request
	sendb = send(camerasocket,get_request,strlen(get_request),0); 
	if (sendb < 0){
		printf("send_request %s\n",strerror(errno));
		return 0;	
	}

	
	if (auth != NULL) send_auth(camerasocket);

	// Finish the request
	sendb = send(camerasocket,"\r\n",2,0);

	if (sendb < 0){
                printf("send_request %s\n",strerror(errno));
                return 0;
        }

	printf("send_request [OK]\n");
	
	return 1;	
}


/*
 *  Read and parse the stream getting Start/End of JPEG and allocation the
 *  image that will be send to client/s.
 *
 *  Return 0 on error
 *  	   1 on success
 *
 */

static int read_stream(int camerasocket)
{
	char *ptr_buffer=NULL;
	char buffer[4096]={'\0'};
	size_t buffsize = 4096;
	int retread=0,buffer_len=0;
	struct timeval timeout;
	int retry=0;
	
	timeout.tv_sec = 5;
	timeout.tv_usec = 0;
	
	if ((setsockopt(camerasocket, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout))) < 0) {
		printf("setsockopt() in netcam_recv failed ,%s [%d]\n",strerror(errno),errno);
	}


	// Read answer
	do {
		retread = recv(camerasocket, &buffer, buffsize, 0);
		if (retread <= 0) {
			printf("[%d] read_stream ,%s [%d]\n",retry,strerror(errno),errno);
			retry++;
		}	
	} while ( ((errno == EINTR) || (errno == EAGAIN)) && (retry < MAX_RETRY));
	
	if (retread <= 0){
		if (retread == -1){	
			printf("read_stream %s [%d]\n",strerror(errno),errno);
		}		
		return 0;
	}

	buffer_len = retread;

	
	if ((buffer_pos + buffer_len) > MAX_IMAGE_BUFFER){
		buffer_pos = 0;
		printf("Maximun length of buffer reach without get any JPEG\n");
		return 0;	
	}


	memmove(image_buffer+buffer_pos,buffer,buffer_len);
	buffer_pos += buffer_len; 


	/* If image has not been read yet */
	if (!start_jpg){	
		/* Start of JPEG  ff d8 (SOI), 3trd is ff and 4th byte could be e0 ,e1, fe 
		 * JPEG-DCF , APP1 Marker ff e1 (where e1 is the 4th byte of JPEG header ).
		 * JPEG-JFIF , APP0 Marker ff e0 (where e0 is the 4th byte of JPEG header ).
		 * Comment ,  COM ff fe ( that is used by Axis ).
		 * First attemp to find the START JPEG */
		
		if ((ptr_buffer = memmem(image_buffer,buffer_pos,"\xff\xd8",2)) != NULL){		
			buffer_len = buffer_pos - (ptr_buffer-image_buffer);
			start_jpg = 1;
			end_jpg = 0;
			memmove(image_buffer, image_buffer+(ptr_buffer-image_buffer), buffer_len);
			buffer_pos = buffer_len;
		}	
	}	
	
	
	if (!image_complete){
		if ((start_jpg) && (!end_jpg)){
			/* check for  End JPEG  ff d9 (EOI) */	
			if ((ptr_buffer = memmem(image_buffer,buffer_pos,"\xff\xd9",2)) != NULL){ 			
				int diff_ptr_image_buffer = (ptr_buffer - image_buffer) + 2;
				
				start_jpg = 0;
				
				image_complete = 1;
				if (jpeg_image != NULL) {
					free(jpeg_image);
					jpeg_image = NULL;
				}	
			
				image_size = diff_ptr_image_buffer;
				jpeg_image = malloc(image_size);
				
				memcpy(jpeg_image,image_buffer,diff_ptr_image_buffer);
				memmove(image_buffer, image_buffer+diff_ptr_image_buffer, buffer_pos - diff_ptr_image_buffer);
				
				buffer_pos -=  diff_ptr_image_buffer;		
				
			}	
		}
	}	
	
		
	return 1;
}

/*
 * Connect to camera stream
 * 
 * Return 0 on error
 * 	  1 on success
 *
 */ 
static int connect_camera(int *camerasocket)
{
	struct sockaddr_in server;      /* for connect */
	struct addrinfo *res=NULL;      /* for getaddrinfo */
	struct timeval timeout;
	int ret;
	int saveflags;
	int back_err;
	socklen_t len;
	fd_set fd_w;
	struct timeval selecttime;

	timeout.tv_sec = 5;
	timeout.tv_usec = 0;
	
	/* create a new socket */
        if (( *camerasocket = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
		printf("Attempting to create socket,%s [%d]\n",strerror(errno),errno);
		return 0;
        }

	/* lookup the hostname given in the netcam URL */
	if ((ret = getaddrinfo(host_camera, NULL, NULL, &res)) != 0) {
		printf("getaddrinfo() failed ,%s [%d]\n",strerror(errno),errno);
		return 0;
	}

	memset(&server, 0, sizeof(server));
	memcpy(&server, res->ai_addr, sizeof(server));
	freeaddrinfo(res);

	server.sin_family = AF_INET;
	server.sin_port = htons(port_camera);
	
	if ((setsockopt(*camerasocket, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout))) < 0) {
		printf("setsockopt() failed,%s [%d]\n",strerror(errno),errno);
		return 0;
	}

	/*
         * Unfortunately, the SO_RCVTIMEO does not affect the 'connect'
         * call, so we need to be quite a bit more clever.  We want to
         * use timeout, so we set the socket non-blocking and
         * then use a 'select' system call to control the timeout.
         */

        if ((saveflags = fcntl(*camerasocket, F_GETFL, 0)) < 0) {
		printf("fcntl(1) on socket,%s [%d]\n",strerror(errno),errno);
                close(*camerasocket);
		*camerasocket = -1;
		return 0;
	}

	/* Set the socket non-blocking */
	if (fcntl(*camerasocket, F_SETFL, saveflags | O_NONBLOCK) < 0) {
		printf("fcntl(2) on socket,%s [%d]\n",strerror(errno),errno);
		close(*camerasocket);
		*camerasocket = -1;
		return 0;
	}

	/* Now the connect call will return immediately */
	ret = connect(*camerasocket, (struct sockaddr *) &server, sizeof(server));
        back_err = errno;           /* save the errno from connect */

        /* Restore the normal socket flags */
        if (fcntl(*camerasocket, F_SETFL, saveflags) < 0) {
                printf("fcntl(3) on socket,%s [%d]\n",strerror(errno),errno);
                close(*camerasocket);
                *camerasocket = -1;
                return 0;
	}	

	
        /* If the connect failed with anything except EINPROGRESS, error */
        if ((ret < 0) && (back_err != EINPROGRESS)) {
                printf("connect() failed (%d),%s [%d]\n", back_err, strerror(errno),errno);
                close(*camerasocket);
                *camerasocket = -1;
                return 0;
        }
					
	/* Now we do a 'select' with timeout to wait for the connect */
        FD_ZERO(&fd_w);
        FD_SET(*camerasocket, &fd_w);
        selecttime.tv_sec = 5;
        selecttime.tv_usec = 0;
        ret = select(FD_SETSIZE, NULL, &fd_w, NULL, &selecttime);

        if (ret == 0) {            /* 0 means timeout */
                printf("timeout on connect(),%s [%d]\n", strerror(errno),errno);
                close(*camerasocket);
                *camerasocket = -1;
                return 0;
        }

        /*
         * A +ve value returned from the select (actually, it must be a
         * '1' showing 1 fd's changed) shows the select has completed.
         * Now we must check the return code from the select.
         */
        len = sizeof(ret);

        if (getsockopt(*camerasocket, SOL_SOCKET, SO_ERROR, &ret, &len) < 0) {
                printf("getsockopt after connect,%s [%d]\n", strerror(errno),errno);
                return 0;
        }

	/* If the return code is anything except 0, error on connect */
        if (ret) {
                printf("connect returned error,%s [%d]\n", strerror(errno),errno);
                return 0;
        }
	
	printf("connect_camera [OK]\n");
		
	return 1;	
}


/*
 *  reconnect
 *
 *  keeps looping trying to connect and send http request to camera
 *  until success
 *
 */ 

static void reconnect(int *camerasocket)
{
	int ok = 0;

	while ( !ok ){
		ok = connect_camera(camerasocket);
		if (ok){
			if (send_request(*camerasocket)) break;
		}else{
			if (*camerasocket != -1) {
			       	close(*camerasocket);
				*camerasocket = -1;
			}	
		}
		
		printf("Failed connect and send_request to [%s] port [%d]\nRetrying\n",host_camera,port_camera);
		// Wait 10 seconds
		SLEEP(10,0);
	}
}

/*
 *	acceptnonblocking
 *
 *	This function waits timeout seconds for listen socket.
 *	Returns :
 *	-1 if the timeout expires or on accept error.
 *	curfd (client socket) on accept success.
 */

static int acceptnonblocking(int serverfd, int timeout)
{
	int curfd;
	socklen_t namelen = sizeof(struct sockaddr_in);
	struct sockaddr_in client;
	struct timeval tm;
	fd_set fds;

	tm.tv_sec = timeout; /* Timeout in seconds */
	tm.tv_usec = 0;
	FD_ZERO(&fds);
	FD_SET(serverfd,&fds);
	
	if( select (serverfd+1, &fds, NULL, NULL, &tm) > 0){
		if(FD_ISSET(serverfd, &fds)) {
			if((curfd = accept(serverfd, (struct sockaddr*)&client, &namelen))>0){
				return(curfd);
			}
		}
	}
	return -1;
}


/*
 * Parse de parameter passed for host
 * 
 * i.ex :
 * -url=http://192.168.1.2:8080/cgi-bin/Stream?Video
 *
 *  host_camera = 192.168.1.2
 *  port_camera = 8080
 *  url_path = /cgi-bin/Stream?Video
 */ 
static int parse_url(char *host)
{
	char *ptr=NULL;
	char *ptr2=NULL;
	
	if ((ptr = strstr(host,"http://")) != NULL){
		int len_host = strlen(host);
		memmove(host,host+7,len_host-7);
		host[len_host-7]='\0';
	}
	
	if ((ptr = strstr(host,":")) != NULL){
		char port_value[6]={'\0'};
		int ret;
	
		host[ptr-host]='\0';	
		host_camera = malloc((strlen(host))+1);

		strncpy(host_camera,host,strlen(host));
		host_camera[(strlen(host))+1]='\0';
		// get port 
		ptr++;
		ret = sscanf(ptr,"%10[0-9]",port_value);
		if (!ret){
			printf("parse_url() *Error* - malformed argument -url PORT not found -\n");
			return 0;
		}
		port_camera = atoi(port_value);	
	}else if ((ptr=strstr(host,"/")) != NULL){
		host[ptr-host]='\0';
		host_camera = malloc((strlen(host))+1);
		strncpy(host_camera,host,(strlen(host)));
		host_camera[(strlen(host))+1]='\0';
		port_camera = 80;
	}else{
		printf("parse_url() *Error* - malformed argument -url HOST not found -\n");
		return 0;
	}

	if ((ptr2 = strstr(ptr,"/")) != NULL){
		get_request = malloc(strlen(get)+strlen(ptr2)+1);
		sprintf(get_request,get,ptr2);
	}else{
		get_request = malloc(strlen(get)+2);
		sprintf(get_request,get,"/");
	}

	return 1;
}

static void help(void)
{
	printf("\n\n\trelay-nc800 [%s]\n\n",VERSION);
	printf("\tUsage: ./relay-nc800 -url=http://192.168.1.2:8080/cgi-bin/Stream?Video -port=15000 [-auth=user:pass] [-debug]\n\n");
	printf("\t-url   : Url of mjpg stream\n");
	printf("\t-port  : port to bind relay-nc800\n");
	printf("\t-auth  : user and pass if needed\n");
	printf("\t-debug : verbose logs\n\n");
}

int main(int argc , char **argv)
{
	int listensocket=-1,camerasocket=-1,clientsocket=-1;
	int val = 1;
	int retry=0;	
	struct sockaddr_in servAddr;
	struct sigaction act;
	int num_frames=0,num_frames_client=0;
	struct timeval time_init,time_end;
	struct timeval time_init_client,time_end_client;
	double fps,fps_client;
		
	// check the parameters at least -host have to be set
	if (argc > 1){
		char *url=NULL;	
		char *port_=NULL;
		if  ((argc >=4 ) && (auth=strstr(argv[3],"-auth=")) != NULL){
			int len_auth;
			len_auth = strlen(auth);
			memmove(auth,auth+6,len_auth-6);
			auth[len_auth-6]='\0';			
		}

		if ( ((argc >=4 ) && (strstr(argv[3],"-debug"))) || ((argc >= 5) && (strstr(argv[4],"-debug"))) ){
			debug = 1;
		}
		
		if ((argc>=3) && ((url=strstr(argv[1],"-url=")) != NULL) ){
			int len_url;
			len_url = strlen(url);
			memmove(url,url+5,len_url-5);
			url[len_url-5]='\0';
			
			if (!parse_url(url)){
				exit(-1);
			}

			if ((port_=strstr(argv[2],"-port=")) != NULL){
				int len_port;
				len_port = strlen(port_);
				memmove(port_,port_+6,len_port-6);
				port_[len_port]='\0';
				port_relay = atoi(port_);
			}else{
				help();
				exit(0);	
			}
		}else{
			help();
			exit(0);
		}

	}else{
		help();
		exit(0);
	}
	
	/* set signal handlers TO IGNORE */
	memset(&act,0,sizeof(act));
	sigemptyset(&act.sa_mask);
	act.sa_handler = SIG_IGN;
	sigaction(SIGPIPE,&act,NULL);
		

	
	/* create socket */
        listensocket = socket(AF_INET, SOCK_STREAM, 0);
	if (listensocket<0) {
		printf("httpd socket error,%s [%d]\n", strerror(errno),errno);
		free(get_request);
		free(host_camera);	
		return -1;
	}

	/* bind server port */
	servAddr.sin_family = AF_INET;
//	servAddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK); // 127.0.0.1
	servAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servAddr.sin_port = htons(port_relay);

	/* Reuse Address */

	setsockopt(listensocket, SOL_SOCKET, SO_REUSEADDR, &val, sizeof( int ) ) ;

	/* Bind socket */
	if (bind(listensocket, (struct sockaddr *) &servAddr, sizeof(servAddr))<0) {
		printf("httpd bind() error,%s [%d]\n", strerror(errno),errno);	
		free(get_request);
		free(host_camera);
		close(listensocket);
		return -1;
	}


	if (listen(listensocket,5) == -1){
		printf("httpd listen(),%s [%d]\n", strerror(errno),errno);			
		close(listensocket);
		free(get_request);
		free(host_camera);
		return -1;
	}	

	if ( connect_camera(&camerasocket) ){
		printf("Connected to [%s] port [%d]\n",host_camera,port_camera);
	}else{
		// retry to connect to camera if first time didn't work
		while ((!connect_camera(&camerasocket)) && (retry < MAX_RETRY)){
			SLEEP(2,0);
			retry++;
			printf("retrying connection [%d]...\n",retry);
		}
	}	
	
	if (retry == MAX_RETRY){
		printf("Error Fatal trying to connect\n");	
		free(get_request);
		free(host_camera);
		return -1;
	}
		
	if (!send_request(camerasocket)) {
		printf("Error sending request ... \n");
		free(get_request);
		free(host_camera);
		return -1;
	}

		
	size_header = strlen(jpeghead);
	
	while(1){
		
		if (!image_complete) {
			if (!read_stream(camerasocket)){
				// Close camera socket a retry the connection
				close(camerasocket);
				camerasocket = -1;
			
				// try to reconnect to camera and keep loop there until success
				reconnect(&camerasocket);
				
				printf("Re-Connected to [%s] port [%d]\n",host_camera,port_camera);
				image_complete = 0;
			}
			if (debug){
				num_frames++;	
				if (num_frames % 25 == 0){		
					double t1,t2,t;
					gettimeofday(&time_end,NULL);
					t1 = time_end.tv_sec+(time_end.tv_usec/1000000.0);
					t2 = time_init.tv_sec+(time_init.tv_usec/1000000.0);	 
					t = t1-t2;
					fps = num_frames / t;
					printf("FPS GOT [%.2lf]\n",fps);
					num_frames=0;
					gettimeofday(&time_init,NULL);
				}	
			}	
		}	
		// Only poll to accept a client if clientsocket hasn't been accepted yet
		if (clientsocket == -1) clientsocket = acceptnonblocking(listensocket, 1);
		
		// only write to client if the image has been read 
		if ((clientsocket != -1) && (image_complete)){
			
			// write stream start header when nothing has been written to client yet	
			if (start_stream) 
				if (!write_ini_header(clientsocket)){
					printf("Error sending header\n");
					return -1;
				}
			image_complete = 0;
			if (!write_client(clientsocket)){
				close(clientsocket);
				clientsocket = -1;
				start_stream = 1;
				printf("Client Disconnected\n");
			}
			if (debug){
				num_frames_client++;
				
				if (num_frames_client % 25 == 0){
					double t1,t2,t;
					gettimeofday(&time_end_client,NULL);
					t1 = time_end_client.tv_sec+(time_end_client.tv_usec/1000000.0);
					t2 = time_init_client.tv_sec+(time_init_client.tv_usec/1000000.0);
					t = t1-t2;
					fps_client = num_frames_client / t;
					printf("FPS Served [%.2lf]\n",fps_client);
					num_frames_client=0;
					gettimeofday(&time_init_client,NULL);
				}
			}	
		}	
	}	
	
	
	return 0;	
}
